@@ -6,31 +6,57 @@ use std::{
     sync::{Arc, Mutex},
 };
 
+use tracing::{instrument, trace};
+
 use crate::{
     endpoints::{default_endpoints, Endpoint},
     errors::{AtomicError, AtomicResult},
     resources::PropVals,
-    storelike::{ResourceCollection, Storelike},
+    storelike::{Query, QueryResult, ResourceCollection, Storelike},
     Atom, Resource, Value,
 };
 
-/// Inside the index_vals, each value is mapped to this type.
+use self::query_index::{
+    atom_to_indexable_atoms, check_if_atom_matches_watched_query_filters, query_indexed,
+    update_indexed_member, watch_collection, IndexAtom, QueryFilter, END_CHAR,
+};
+
+mod query_index;
+#[cfg(test)]
+pub mod test;
+
+/// Inside the reference_index, each value is mapped to this type.
 /// The String on the left represents a Property URL, and the second one is the set of subjects.
 pub type PropSubjectMap = HashMap<String, HashSet<String>>;
 
 /// The Db is a persistent on-disk Atomic Data store.
-/// It's an implementation of Storelike.
+/// It's an implementation of [Storelike].
+/// It uses [sled::Tree]s as Key Value stores.
+/// It builds a value index for performant [Query]s.
+/// It stores [Resource]s as [PropVals]s by their subject as key.
+///
+/// ## Resources
+///
+/// ## Value Index
+///
+/// The Value index stores
 #[derive(Clone)]
 pub struct Db {
-    // The Key-Value store that contains all data.
-    // Resources can be found using their Subject.
-    // Try not to use this directly, but use the Trees.
+    /// The Key-Value store that contains all data.
+    /// Resources can be found using their Subject.
+    /// Try not to use this directly, but use the Trees.
     db: sled::Db,
     default_agent: Arc<Mutex<Option<crate::agents::Agent>>>,
-    // Stores all resources. The Key is the Subject as a string, the value a PropVals. Both must be serialized using bincode.
+    /// Stores all resources. The Key is the Subject as a string, the value a [PropVals]. Both must be serialized using bincode.
     resources: sled::Tree,
-    // Stores all Atoms. The key is the atom.value, the value a vector of Atoms.
-    index_vals: sled::Tree,
+    /// Index for all AtommicURLs, indexed by their Value. Used to speed up TPF queries. See [key_for_reference_index]
+    reference_index: sled::Tree,
+    /// Stores the members of Collections, easily sortable.
+    /// See [collections_index]
+    members_index: sled::Tree,
+    /// A list of all the Collections currently being used. Is used to update `members_index`.
+    /// See [collections_index]
+    watched_queries: sled::Tree,
     /// The address where the db will be hosted, e.g. http://localhost/
     server_url: String,
     /// Endpoints are checked whenever a resource is requested. They calculate (some properties of) the resource and return it.
@@ -44,13 +70,17 @@ impl Db {
     pub fn init(path: &std::path::Path, server_url: String) -> AtomicResult<Db> {
         let db = sled::open(path).map_err(|e|format!("Failed opening DB at this location: {:?} . Is another instance of Atomic Server running? {}", path, e))?;
         let resources = db.open_tree("resources").map_err(|e|format!("Failed building resources. Your DB might be corrupt. Go back to a previous version and export your data. {}", e))?;
-        let index_vals = db.open_tree("index_vals")?;
+        let reference_index = db.open_tree("reference_index")?;
+        let members_index = db.open_tree("members_index")?;
+        let watched_queries = db.open_tree("watched_queries")?;
         let store = Db {
             db,
             default_agent: Arc::new(Mutex::new(None)),
             resources,
-            index_vals,
+            reference_index,
+            members_index,
             server_url,
+            watched_queries,
             endpoints: default_endpoints(),
         };
         crate::populate::populate_base_models(&store)
@@ -58,8 +88,23 @@ impl Db {
         Ok(store)
     }
 
+    /// Create a temporary Db in `.temp/db/{id}`. Useful for testing.
+    /// Populates the database, creates a default agent, and sets the server_url to "http://localhost/".
+    pub fn init_temp(id: &str) -> AtomicResult<Db> {
+        let tmp_dir_path = format!(".temp/db/{}", id);
+        let _try_remove_existing = std::fs::remove_dir_all(&tmp_dir_path);
+        let store = Db::init(
+            std::path::Path::new(&tmp_dir_path),
+            "https://localhost".into(),
+        )?;
+        let agent = store.create_agent(None)?;
+        store.set_default_agent(agent);
+        store.populate()?;
+        Ok(store)
+    }
+
     /// Internal method for fetching Resource data.
-    #[tracing::instrument(skip(self))]
+    #[instrument(skip(self))]
     fn set_propvals(&self, subject: &str, propvals: &PropVals) -> AtomicResult<()> {
         let resource_bin = bincode::serialize(propvals)?;
         let subject_bin = bincode::serialize(subject)?;
@@ -69,7 +114,7 @@ impl Db {
 
     /// Finds resource by Subject, return PropVals HashMap
     /// Deals with the binary API of Sled
-    #[tracing::instrument(skip(self))]
+    #[instrument(skip(self))]
     fn get_propvals(&self, subject: &str) -> AtomicResult<PropVals> {
         let subject_binary = bincode::serialize(subject)
             .map_err(|e| format!("Can't serialize {}: {}", subject, e))?;
@@ -97,17 +142,20 @@ impl Db {
 
     /// Returns true if the index has been built.
     pub fn has_index(&self) -> bool {
-        !self.index_vals.is_empty()
+        !self.reference_index.is_empty()
     }
 
-    /// Removes all values from the index.
+    /// Removes all values from the indexes.
     pub fn clear_index(&self) -> AtomicResult<()> {
-        self.index_vals.clear()?;
+        self.reference_index.clear()?;
+        self.members_index.clear()?;
+        self.watched_queries.clear()?;
         Ok(())
     }
 }
 
 impl Storelike for Db {
+    #[instrument(skip(self))]
     fn add_atoms(&self, atoms: Vec<Atom>) -> AtomicResult<()> {
         // Start with a nested HashMap, containing only strings.
         let mut map: HashMap<String, Resource> = HashMap::new();
@@ -136,28 +184,21 @@ impl Storelike for Db {
         Ok(())
     }
 
-    // This only adds ResourceArrays and AtomicURLs at this moment, which means that many values cannot be accessed in the TPF query (thus, collections)
-    #[tracing::instrument(skip(self))]
-    fn add_atom_to_index(&self, atom: &Atom) -> AtomicResult<()> {
-        let values_vec = match &atom.value {
-            // This results in wrong indexing, as some subjects will be numbers.
-            Value::ResourceArray(_v) => atom.values_to_subjects()?,
-            Value::AtomicUrl(v) => vec![v.into()],
-            _other => return Ok(()),
-        };
-
-        for val_subject in values_vec {
-            // https://github.com/joepio/atomic-data-rust/issues/282
-            // The key is a newline delimited string, with the following format:
-            let key = key_for_value_index(&val_subject, &atom.property, &atom.subject);
-
+    #[instrument(skip(self))]
+    fn add_atom_to_index(&self, atom: &Atom, resource: &Resource) -> AtomicResult<()> {
+        for index_atom in atom_to_indexable_atoms(atom)? {
             // It's OK if this overwrites a value
-            let _existing = self.index_vals.insert(key.as_bytes(), b"")?;
+            add_atom_to_reference_index(&index_atom, self)?;
+            // Also update the query index to keep collections performant
+            check_if_atom_matches_watched_query_filters(self, &index_atom, atom, false, resource)
+                .map_err(|e| {
+                    format!("Failed to check_if_atom_matches_watched_collections. {}", e)
+                })?;
         }
         Ok(())
     }
 
-    #[tracing::instrument(skip(self, resource), fields(sub = %resource.get_subject()))]
+    #[instrument(skip(self, resource), fields(sub = %resource.get_subject()))]
     fn add_resource_opts(
         &self,
         resource: &Resource,
@@ -184,27 +225,27 @@ impl Storelike for Db {
                 for (prop, val) in pv.iter() {
                     // Possible performance hit - these clones can be replaced by modifying remove_atom_from_index
                     let remove_atom = crate::Atom::new(subject.into(), prop.into(), val.clone());
-                    self.remove_atom_from_index(&remove_atom)?;
+                    self.remove_atom_from_index(&remove_atom, resource)
+                        .map_err(|e| {
+                            format!("Failed to remove atom from index {}. {}", remove_atom, e)
+                        })?;
                 }
             }
             for a in resource.to_atoms()? {
-                self.add_atom_to_index(&a)?;
+                self.add_atom_to_index(&a, resource)
+                    .map_err(|e| format!("Failed to add atom to index {}. {}", a, e))?;
             }
         }
         self.set_propvals(resource.get_subject(), resource.get_propvals())
     }
 
-    #[tracing::instrument(skip(self))]
-    fn remove_atom_from_index(&self, atom: &Atom) -> AtomicResult<()> {
-        let vec = match atom.value.to_owned() {
-            Value::ResourceArray(_v) => atom.values_to_subjects()?,
-            Value::AtomicUrl(subject) => vec![subject],
-            _other => return Ok(()),
-        };
+    #[instrument(skip(self))]
+    fn remove_atom_from_index(&self, atom: &Atom, resource: &Resource) -> AtomicResult<()> {
+        for index_atom in atom_to_indexable_atoms(atom)? {
+            delete_atom_from_reference_index(&index_atom, self)?;
 
-        for val in vec {
-            let key = key_for_value_index(&val, &atom.property, &atom.subject);
-            self.index_vals.remove(&key.as_bytes())?;
+            check_if_atom_matches_watched_query_filters(self, &index_atom, atom, true, resource)
+                .map_err(|e| format!("Checking atom went wrong: {}", e))?;
         }
         Ok(())
     }
@@ -226,7 +267,7 @@ impl Storelike for Db {
         }
     }
 
-    #[tracing::instrument(skip(self))]
+    #[instrument(skip(self))]
     fn get_resource(&self, subject: &str) -> AtomicResult<Resource> {
         let propvals = self.get_propvals(subject);
 
@@ -239,14 +280,14 @@ impl Storelike for Db {
         }
     }
 
-    #[tracing::instrument(skip(self))]
+    #[instrument(skip(self))]
     fn get_resource_extended(
         &self,
         subject: &str,
         skip_dynamic: bool,
         for_agent: Option<&str>,
     ) -> AtomicResult<Resource> {
-        tracing::trace!("get_resource_extended: {}", subject);
+        trace!("get_resource_extended: {}", subject);
         // This might add a trailing slash
         let mut url = url::Url::parse(subject)?;
         let clone = url.clone();
@@ -284,7 +325,7 @@ impl Storelike for Db {
         resource.set_subject(subject.into());
 
         if let Some(agent) = for_agent {
-            crate::hierarchy::check_read(self, &resource, agent)?;
+            let _explanation = crate::hierarchy::check_read(self, &resource, agent)?;
         }
 
         // Whether the resource has dynamic properties
@@ -335,7 +376,96 @@ impl Storelike for Db {
         Ok(resource)
     }
 
-    #[tracing::instrument(skip(self))]
+    /// Search the Store, returns the matching subjects.
+    /// The second returned vector should be filled if query.include_resources is true.
+    /// Tries `query_cache`, which you should implement yourself.
+    #[instrument(skip(self))]
+    fn query(&self, q: &Query) -> AtomicResult<QueryResult> {
+        if let Ok(res) = query_indexed(self, q) {
+            if res.count > 0 {
+                // Yay, we have a cache hit!
+                // We don't have to perform a (more expansive) TPF query + sorting
+                return Ok(res);
+            }
+        }
+
+        // No cache hit, perform the query
+        let mut atoms = self.tpf(
+            None,
+            q.property.as_deref(),
+            q.value.as_ref(),
+            // We filter later on, not here
+            true,
+        )?;
+        let count = atoms.len();
+
+        let mut subjects = Vec::new();
+        let mut resources = Vec::new();
+        for atom in atoms.iter() {
+            // These nested resources are not fully calculated - they will be presented as -is
+            subjects.push(atom.subject.clone());
+            // We need the Resources if we want to sort by a non-subject value
+            if q.include_nested || q.sort_by.is_some() {
+                // We skip checking for Agent, because we don't return these results directly anyway
+                match self.get_resource_extended(&atom.subject, true, None) {
+                    Ok(resource) => {
+                        resources.push(resource);
+                    }
+                    Err(e) => match e.error_type {
+                        crate::AtomicErrorType::NotFoundError => {}
+                        crate::AtomicErrorType::UnauthorizedError => {}
+                        crate::AtomicErrorType::OtherError => {
+                            return Err(
+                                format!("Error when getting resource in collection: {}", e).into()
+                            )
+                        }
+                    },
+                }
+            }
+        }
+
+        if atoms.is_empty() {
+            return Ok(QueryResult {
+                subjects: vec![],
+                resources: vec![],
+                count,
+            });
+        }
+
+        // If there is a sort value, we need to change the items to contain that sorted value, instead of the one matched in the TPF query
+        if let Some(sort) = &q.sort_by {
+            // We don't use the existing array, we clear it.
+            atoms = Vec::new();
+            for r in &resources {
+                // Users _can_ sort by optional properties! So we need a fallback defauil
+                let fallback_default = crate::Value::String(END_CHAR.into());
+                let sorted_val = r.get(sort).unwrap_or(&fallback_default);
+                let atom = Atom {
+                    subject: r.get_subject().to_string(),
+                    property: sort.to_string(),
+                    value: sorted_val.to_owned(),
+                };
+                atoms.push(atom)
+            }
+            // Now we sort by the value that the user wants to sort by
+            atoms.sort_by(|a, b| a.value.to_string().cmp(&b.value.to_string()));
+        }
+
+        let q_filter: QueryFilter = q.into();
+
+        // Maybe make this optional?
+        watch_collection(self, &q_filter)?;
+
+        // Add the atoms to the query_index
+        for atom in atoms {
+            update_indexed_member(self, &q_filter, &atom, false)?;
+        }
+
+        // Retry the same query!
+        query_indexed(self, q)
+    }
+
+    #[instrument(skip(self))]
     fn all_resources(&self, include_external: bool) -> ResourceCollection {
         let mut resources: ResourceCollection = Vec::new();
         let self_url = self
@@ -370,14 +500,15 @@ impl Storelike for Db {
         Ok(())
     }
 
-    #[tracing::instrument(skip(self))]
+    #[instrument(skip(self))]
     fn remove_resource(&self, subject: &str) -> AtomicResult<()> {
         if let Ok(found) = self.get_propvals(subject) {
-            for (prop, val) in found {
-                let remove_atom = crate::Atom::new(subject.into(), prop, val);
-                self.remove_atom_from_index(&remove_atom)?;
+            let resource = Resource::from_propvals(found, subject.to_string());
+            for (prop, val) in resource.get_propvals() {
+                let remove_atom = crate::Atom::new(subject.into(), prop.clone(), val.clone());
+                self.remove_atom_from_index(&remove_atom, &resource)?;
             }
-            let binary_subject = bincode::serialize(subject).unwrap();
+            let binary_subject = bincode::serialize(subject)?;
             let _found = self.resources.remove(&binary_subject)?;
         } else {
             return Err(format!(
@@ -394,16 +525,16 @@ impl Storelike for Db {
     }
 
     // TPF implementation that used the index_value cache, far more performant than the StoreLike implementation
-    #[tracing::instrument(skip(self))]
+    #[instrument(skip(self))]
     fn tpf(
         &self,
         q_subject: Option<&str>,
         q_property: Option<&str>,
-        q_value: Option<&str>,
+        q_value: Option<&Value>,
         // Whether resources from outside the store should be searched through
         include_external: bool,
     ) -> AtomicResult<Vec<Atom>> {
-        tracing::trace!("tpf");
+        trace!("tpf");
         let mut vec: Vec<Atom> = Vec::new();
 
         let hassub = q_subject.is_some();
@@ -426,11 +557,11 @@ impl Storelike for Db {
 
         // If the value is a resourcearray, check if it is inside
         let val_equals = |val: &str| {
-            let q = q_value.unwrap();
+            let q = q_value.unwrap().to_sortable_string();
             val == q || {
                 if val.starts_with('[') {
                     match crate::parse::parse_json_array(val) {
-                        Ok(vec) => return vec.contains(&q.into()),
+                        Ok(vec) => return vec.contains(&q),
                         Err(_) => return val == q,
                     }
                 }
@@ -477,10 +608,12 @@ impl Storelike for Db {
                     } else {
                         format!("{}\n", q_value.unwrap())
                     };
-                    for item in self.index_vals.scan_prefix(key_prefix) {
+                    for item in self.reference_index.scan_prefix(key_prefix) {
                         let (k, _v) = item?;
                         let key_string = String::from_utf8(k.to_vec())?;
+                        // WARNING: Converts all Atoms to Strings, the datatype is lost here
                         let atom = key_to_atom(&key_string)?;
+                        // NOTE: This means we'll include random values that start with the current server URL, including paragraphs for example.
                         if include_external || atom.subject.starts_with(self.get_server_url()) {
                             vec.push(atom)
                         }
@@ -497,11 +630,28 @@ impl Storelike for Db {
     }
 }
 
-fn key_for_value_index(val: &str, prop: &str, subject: &str) -> String {
-    format!("{}\n{}\n{}", val, prop, subject)
+#[instrument(skip(store))]
+fn add_atom_to_reference_index(index_atom: &IndexAtom, store: &Db) -> AtomicResult<()> {
+    let _existing = store
+        .reference_index
+        .insert(key_for_reference_index(index_atom).as_bytes(), b"")?;
+    Ok(())
 }
 
-/// Parses a Value index key string, converts it into an atom. Note that the Value of the atom will allways be a string here.
+#[instrument(skip(store))]
+fn delete_atom_from_reference_index(index_atom: &IndexAtom, store: &Db) -> AtomicResult<()> {
+    store
+        .reference_index
+        .remove(&key_for_reference_index(index_atom).as_bytes())?;
+    Ok(())
+}
+
+/// Constructs the Key for the index_value cache.
+fn key_for_reference_index(atom: &IndexAtom) -> String {
+    format!("{}\n{}\n{}", atom.value, atom.property, atom.subject)
+}
+
+/// Parses a Value index key string, converts it into an atom. Note that the Value of the atom will allways be a single AtomicURL here.
 fn key_to_atom(key: &str) -> AtomicResult<Atom> {
     let mut parts = key.split('\n');
     let val = parts.next().ok_or("Invalid key for value index")?;
@@ -519,250 +669,3 @@ fn corrupt_db_message(subject: &str) -> String {
 }
 
 const DB_CORRUPT_MSG: &str = "Could not deserialize item from database. DB is possibly corrupt, could be due to an update or a lack of migrations. Restore to a previous version, export / serialize your data and import your data again.";
-
-#[cfg(test)]
-pub mod test {
-    use crate::urls;
-
-    use super::*;
-    use ntest::timeout;
-
-    /// Creates new temporary database, populates it, removes previous one.
-    /// Can only be run one thread at a time, because it requires a lock on the DB file.
-    fn init(id: &str) -> Db {
-        let tmp_dir_path = format!("tmp/db/{}", id);
-        let _try_remove_existing = std::fs::remove_dir_all(&tmp_dir_path);
-        let store = Db::init(
-            std::path::Path::new(&tmp_dir_path),
-            "https://localhost".into(),
-        )
-        .unwrap();
-        let agent = store.create_agent(None).unwrap();
-        store.set_default_agent(agent);
-        store.populate().unwrap();
-        store
-    }
-
-    /// Share the Db instance between tests. Otherwise, all tests try to init the same location on disk and throw errors.
-    /// Note that not all behavior can be properly tested with a shared database.
-    /// If you need a clean one, juts call init("someId").
-    use lazy_static::lazy_static; // 1.4.0
-    use std::sync::Mutex;
-    lazy_static! {
-        pub static ref DB: Mutex<Db> = Mutex::new(init("shared"));
-    }
-
-    #[test]
-    #[timeout(30000)]
-    fn basic() {
-        let store = DB.lock().unwrap().clone();
-        // We can create a new Resource, linked to the store.
-        // Note that since this store only exists in memory, it's data cannot be accessed from the internet.
-        // Let's make a new Property instance!
-        let mut new_resource =
-            crate::Resource::new_instance("https://atomicdata.dev/classes/Property", &store)
-                .unwrap();
-        // And add a description for that Property
-        new_resource
-            .set_propval_shortname("description", "the age of a person", &store)
-            .unwrap();
-        new_resource
-            .set_propval_shortname("shortname", "age", &store)
-            .unwrap();
-        new_resource
-            .set_propval_shortname("datatype", crate::urls::INTEGER, &store)
-            .unwrap();
-        // Changes are only applied to the store after saving them explicitly.
-        new_resource.save_locally(&store).unwrap();
-        // The modified resource is saved to the store after this
-
-        // A subject URL has been created automatically.
-        let subject = new_resource.get_subject();
-        let fetched_new_resource = store.get_resource(subject).unwrap();
-        let description_val = fetched_new_resource
-            .get_shortname("description", &store)
-            .unwrap()
-            .to_string();
-        assert!(description_val == "the age of a person");
-
-        // Try removing something
-        store.get_resource(crate::urls::CLASS).unwrap();
-        store.remove_resource(crate::urls::CLASS).unwrap();
-        // Should throw an error, because can't remove non-existent resource
-        store.remove_resource(crate::urls::CLASS).unwrap_err();
-        // Should throw an error, because resource is deleted
-        store.get_propvals(crate::urls::CLASS).unwrap_err();
-
-        assert!(store.all_resources(false).len() < store.all_resources(true).len());
-    }
-
-    #[test]
-    fn populate_collections() {
-        let store = DB.lock().unwrap().clone();
-        let subjects: Vec<String> = store
-            .all_resources(false)
-            .into_iter()
-            .map(|r| r.get_subject().into())
-            .collect();
-        println!("{:?}", subjects);
-        let collections_collection_url = format!("{}/collections", store.get_server_url());
-        let collections_resource = store
-            .get_resource_extended(&collections_collection_url, false, None)
-            .unwrap();
-        let member_count = collections_resource
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        assert!(member_count > 11);
-        let nested = collections_resource
-            .get(crate::urls::COLLECTION_INCLUDE_NESTED)
-            .unwrap()
-            .to_bool()
-            .unwrap();
-        assert!(nested);
-    }
-
-    #[test]
-    /// Check if the cache is working
-    fn add_atom_to_index() {
-        let store = DB.lock().unwrap().clone();
-        let subject = urls::CLASS.into();
-        let property: String = urls::PARENT.into();
-        let val_string = urls::AGENT;
-        let value = Value::new(val_string, &crate::datatype::DataType::AtomicUrl).unwrap();
-        // This atom should normally not exist - Agent is not the parent of Class.
-        let atom = Atom::new(subject, property.clone(), value);
-        store.add_atom_to_index(&atom).unwrap();
-        let found_no_external = store
-            .tpf(None, Some(&property), Some(val_string), false)
-            .unwrap();
-        // Don't find the atom if no_external is true.
-        assert_eq!(
-            found_no_external.len(),
-            0,
-            "found items - should ignore external items"
-        );
-        let found_external = store
-            .tpf(None, Some(&property), Some(val_string), true)
-            .unwrap();
-        // If we see the atom, it's in the index.
-        assert_eq!(found_external.len(), 1);
-    }
-
-    #[test]
-    /// Check if a resource is properly removed from the DB after a delete command.
-    /// Also counts commits.
-    fn destroy_resource_and_check_collection_and_commits() {
-        let store = init("counter");
-        let agents_url = format!("{}/agents", store.get_server_url());
-        let agents_collection_1 = store
-            .get_resource_extended(&agents_url, false, None)
-            .unwrap();
-        let agents_collection_count_1 = agents_collection_1
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        assert_eq!(
-            agents_collection_count_1, 1,
-            "The Agents collection is not one (we assume there is one agent already present from init)"
-        );
-
-        // We will count the commits, and check if they've incremented later on.
-        let commits_url = format!("{}/commits", store.get_server_url());
-        let commits_collection_1 = store
-            .get_resource_extended(&commits_url, false, None)
-            .unwrap();
-        let commits_collection_count_1 = commits_collection_1
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        println!("Commits collection count 1: {}", commits_collection_count_1);
-
-        let mut resource = crate::agents::Agent::new(None, &store)
-            .unwrap()
-            .to_resource(&store)
-            .unwrap();
-        resource.save_locally(&store).unwrap();
-        let agents_collection_2 = store
-            .get_resource_extended(&agents_url, false, None)
-            .unwrap();
-        let agents_collection_count_2 = agents_collection_2
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        assert_eq!(
-            agents_collection_count_2, 2,
-            "The Resource was not found in the collection."
-        );
-
-        let commits_collection_2 = store
-            .get_resource_extended(&commits_url, false, None)
-            .unwrap();
-        let commits_collection_count_2 = commits_collection_2
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        println!("Commits collection count 2: {}", commits_collection_count_2);
-        assert_eq!(
-            commits_collection_count_2,
-            commits_collection_count_1 + 1,
-            "The commits collection did not increase after saving the resource."
-        );
-
-        resource.destroy(&store).unwrap();
-        let agents_collection_3 = store
-            .get_resource_extended(&agents_url, false, None)
-            .unwrap();
-        let agents_collection_count_3 = agents_collection_3
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        assert_eq!(
-            agents_collection_count_3, 1,
-            "The collection count did not decrease after destroying the resource."
-        );
-
-        let commits_collection_3 = store
-            .get_resource_extended(&commits_url, false, None)
-            .unwrap();
-        let commits_collection_count_3 = commits_collection_3
-            .get(crate::urls::COLLECTION_MEMBER_COUNT)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        println!("Commits collection count 3: {}", commits_collection_count_3);
-        assert_eq!(
-            commits_collection_count_3,
-            commits_collection_count_2 + 1,
-            "The commits collection did not increase after destroying the resource."
-        );
-    }
-
-    #[test]
-    fn get_extended_resource_pagination() {
-        let store = DB.lock().unwrap().clone();
-        let subject = format!("{}/commits?current_page=2", store.get_server_url());
-        // Should throw, because page 2 is out of bounds for default page size
-        let _wrong_resource = store
-            .get_resource_extended(&subject, false, None)
-            .unwrap_err();
-        // let subject = "https://atomicdata.dev/classes?current_page=2&page_size=1";
-        let subject_with_page_size = format!("{}&page_size=1", subject);
-        let resource = store
-            .get_resource_extended(&subject_with_page_size, false, None)
-            .unwrap();
-        let cur_page = resource
-            .get(urls::COLLECTION_CURRENT_PAGE)
-            .unwrap()
-            .to_int()
-            .unwrap();
-        assert_eq!(cur_page, 2);
-        assert_eq!(resource.get_subject(), &subject_with_page_size);
-    }
-}
